# Blender add-on. Import in User Settings/Addons.

import bpy
from math import pi

		
bl_info = {
	"name": "OpenTTD/CETS sprite renderer",
	"author": "oberhuemer",
	"blender": (2, 6, 0),
	"version": (0, 0, 1),
	"location": "Render settings/Render for OpenTTD",
	"description": "Automatically render sprites for OpenTTD NewGRFs",
	"category": "Render"}

class ttd_autorender_operator(bpy.types.Operator):
	bl_idname = "bpt.ttd_autorender_operator"	
	bl_label = "Start render"
	
	cur_object = None
	cur_context = None
	
	def render_pass(self, init_z_rot_radians, z_rot_frac_radians, length_scale, zoom):
		self.cur_object.scale = [zoom, zoom * length_scale, zoom]
		self.cur_object.rotation_euler.z = init_z_rot_radians
	
		i = 0
		while  i < 2 * pi / z_rot_frac_radians:
			bpy.ops.render.render()
			# TODO: add livery name? Separate models are easier to handle.
			# Add path for company too (assume all models in certain directory)?
			bpy.data.images['Render Result'].save_render(bpy.path.abspath(self.cur_context.scene.render.filepath) + bpy.path.display_name_from_filepath(bpy.data.filepath) + '-' + str(round(self.cur_object.rotation_euler.z * 180 / pi)) + 'deg.png')
	
			self.cur_object.rotation_euler.z += z_rot_frac_radians
			i += 1
			
		self.cur_object.scale = [1.0, 1.0, 1.0]
		self.cur_object.rotation_euler.z = 0
	
	def execute(self, context):
		self.cur_context = context
		self.cur_context.scene.render.image_settings.file_format = 'PNG'
		self.cur_object = self.cur_context.active_object
		
		if (self.cur_object == None) or (str(self.cur_object.type) != 'MESH'):
			self.report({'ERROR'}, 'Error: A mesh object must be selected')
		else:
			# Standard horizontal/vertical views
			self.render_pass(0, pi / 2, 1.0, 1.0)
	
			# Standard diagonal views
			self.render_pass(pi / 4, pi / 2, 0.75 * 32.0 / 34.0, 1.0)
	
			# Extra CETS angles, sorted from most horizontal to most vertical
			# Angle values found by binary search, scaling by comparison of side lengths
			# Angles kept positive for better ordering
			self.render_pass(pi * 8.38125 / 180, pi, 57.0 / 63.0, 1.0)
			self.render_pass(pi * (1 - 8.38125 / 180), pi, 57.0 / 63.0, 1.0)
			self.render_pass(pi * 30.9375 / 180, pi, 40.0 / 55.0, 1.0)
			self.render_pass(pi * (1 - 30.9375 / 180), pi, 40.0 / 55.0, 1.0)
			self.render_pass(pi * 56.25 / 180, pi, 25.0 / 35.0, 1.0)
			self.render_pass(pi * (1 - 56.25 / 180), pi, 25.0 / 35.0, 1.0)
			self.render_pass(pi * 80.390625 / 180, pi, 0.8425, 1.0)
			self.render_pass(pi * (1 - 80.390625 / 180), pi, 0.8425, 1.0)
			
		return {'FINISHED'}


class ttd_autorender_panel(bpy.types.Panel):
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "render"
	bl_label = "Render for OpenTTD"
	
	def draw(self, context):
		scn = context.scene
		layout = self.layout  
		row = layout.row()
		row.operator("bpt.ttd_autorender_operator")
		



def register():
	bpy.utils.register_class(ttd_autorender_panel)
	bpy.utils.register_class(ttd_autorender_operator)

# ordering is important
def unregister(): 
	bpy.utils.unregister_class(ttd_autorender_operator)
	bpy.utils.unregister_class(ttd_autorender_panel)
	      
if __name__ == "__main__":
	register()
	
	