#!/usr/bin/python
#encoding: utf8
"""
This script reads the tracking table and writes out nml code for the engines contained
"""
from functools import partial

import tables
import project_config
import write
import write_engine
import files
import processing
import read
import helper

read.read(files.tracking_file())

def foreach_vehicle(processes, cmp=None):
	keys = tables.table.keys() # no iterator here, because processing functions might change the table
	if cmp:
		keys.sort(cmp=cmp)
	for key in keys:
		try:
			iter(processes)
		except TypeError:
			processes = [processes]
		for process in processes:
			process(key, **tables.table[key])

foreach_vehicle(processing.unique)
foreach_vehicle(partial(processing.availability, tables.avails))
foreach_vehicle(processing.parts)
foreach_vehicle(processing.recolour)
foreach_vehicle(processing.names)
foreach_vehicle(partial(processing.strings, tables.auto_strings))
foreach_vehicle(processing.capacity)
foreach_vehicle(processing.refits)
foreach_vehicle(processing.lifetime)
foreach_vehicle(processing.conditional_lifetime)
foreach_vehicle(processing.availability2)
foreach_vehicle(partial(processing.misc, project_config.usage_strings, tables.fixed_strings))
foreach_vehicle(partial(processing.track_type, tables.fixed_strings))
foreach_vehicle(processing.artic_props)

# write the files
language_files = [(files.lang_input_file(lang), files.language_file(lang)) for lang in project_config.langs]
table_file = files.table_file()
callbacks_file = files.callbacks_file()
engine_sort_file = files.engine_sort_file()
engines_file = files.engines_file()
defines_file = files.defines_file()
availability_file = files.availability_file()
gfx_deps_file = files.gfx_deps_file()
src_deps_file = files.src_deps_file()
dummy_code_file = files.dummy_code_file()

files.mkdir(*project_config.comps)

for length, slices in tables.parts.iteritems():
	template_file = files.template_file(length)
	write.templates(template_file, length, slices, tables.offsets[length])
	template_file.close()
for lang_input_file, language_file in language_files:
	write.copy(lang_input_file, language_file)
	write.strings(language_file, tables.auto_strings)
write.table_header(table_file, tables.titles, tables.width)
write.global_callbacks(callbacks_file, tables.caps)
write.dummy_code(dummy_code_file, tables.auto_strings, tables.fixed_strings, tables.caps)
write.engine_sort_header(engine_sort_file)
write.availability(availability_file, defines_file, tables.avails, project_config.companies)
foreach_vehicle(partial(write.engine_sort_entry, engine_sort_file), partial(helper.sort_purchase_list, tables.table, project_config.comps))
write.engine_sort_footer(engine_sort_file)
write.engines_header(engines_file)
foreach_vehicle([
		partial(write_engine.vehicle_header, files, tables.files),
		partial(write.engines_entry, engines_file),
		partial(write.table_entry, table_file),
		partial(write.name, language_files),
		partial(write_engine.graphics, tables.gfx_deps),
		write_engine.properties,
		write_engine.articulation,
		write_engine.purchase_text,
		write_engine.callbacks,
		write_engine.vehicle_footer,
	],
	partial(helper.sort_engine_list, project_config.comps))
write.engines_footer(engines_file)
write.gfx_deps(gfx_deps_file, tables.gfx_deps)
write.src_deps(src_deps_file, map(lambda x: '%s.onfo'%(x.rsplit('.gnml',1)[0]), tables.files.itervalues()))

files.compare_files(tables.files.itervalues())

engines_file.close()
language_file.close()
table_file.close()

processing.unused()
