#!/usr/bin/python
#encoding: utf8
"""
This handles the autogeneration of images
"""
from error import warning

try:
	import pixa
	import Image
except ImportError:
	warning("pixa is not installed, automatic sprite processing is disabled")

	def export(outfile, infile, shape):
		return "%s_8bpp_normal.png"%(infile.replace('pixa','template'),)
else:
	import os
	import tables
	import files
	import cPickle as pickle

	def export(outfile, infile, shape):
		templatefile = "%s_8bpp_normal.png"%(infile,)
		replacements = shape.items()
		pixafile = "%s_8bpp_normal.pixa"%(outfile)
		imagefile = "%s.png"%(pixafile,)
		dumpfile = files.pixa_file_out(pixafile)
		pickle.dump((templatefile,replacements, imagefile), dumpfile)
		dumpfile.close()
		return imagefile

	def encode(filename):
		dumpfile = files.pixa_file_in(filename)
		templatefile, replacements, imagefile = pickle.load(dumpfile)
		dumpfile.close()
		floorplan = Image.open(templatefile)
		spritesheet = pixa.Spritesheet(floorplan.size[0], floorplan.size[1], floorplan.palette)
		spriterow = {'height' : floorplan.size[1], 'floorplan' : floorplan}
		seq = {}
		for key, value in replacements:
			seq[key] = pixa.PixaSequence(points = value)
		sc = pixa.PixaSequenceCollection(sequences = seq)
		spriterow['render_passes'] = [{'seq':sc, 'colourset':None}]
		spritesheet.render(spriterows=[spriterow])
		spritesheet.save(imagefile)

if __name__ == "__main__":
	import sys

	if len(sys.argv) == 1:
		sys.stderr.write("Process pickled sequences with pixa\n")
		sys.stderr.write("Usage: img.py pixafile[, pixafile, ...]\n")
		sys.exit(1)

	for filename in sys.argv[1:]:
		encode(filename)
