#!/usr/bin/python
#encoding: utf8
"""
This file contains the function(s) reading the tracking table and preparing it for processing
"""

import tables
import project_config

def read(file):
	max_id = 0

	# Open the tracking table
	# export it from google-docs by selecting File => Download As => Text
	# this creates a tab-separated-values table (tsv)

	file.next() # first line contains rules etc., nothing interesting here
	tables.titles = map(str, file.next().strip('\n').split('\t')) # second line contains column titles
	for i, name in enumerate(file.next().strip('\n').split('\t')): # third line contains column identifiers
		if name:
			tables.columns[tables.titles[i]] = str(name)
	tables.width = [len(title) for title in tables.titles]

	# create a table
	# read the tracking table line-by-line
	# write the entries into a dictonary
	# row-key is the (company, identifier) pair
	# column-key is the column title (as read from the 2nd line of the table above)
	# stop if you encounter an empty line (start of legend)
	for line in file:
		line = line.strip('\n').split('\t') # split up fields by tab
		try: # read the company/identifier of the entry
			comp  = line[tables.titles.index('Company')].split(',')[0] # column contains company, comma separated. take the first entry
			if not comp in project_config.comps:
				project_config.comps.append(comp)
			ident = line[0]
		except IndexError:
			break # if something fails, this line is considered the end of the table
		if comp: # if entry has empty company field, ignore it
			entry = {} # create new row in the dictionary
			entry["values"] = entry # self-reference
			for i, name in enumerate(tables.titles):
				try:
					entry[tables.columns.get(name, name)] = line[i] # fill column/value pairs
					tables.width[i] = max(tables.width[i], len(line[i]))
				except IndexError:
					entry[tables.columns.get(name, name)] = '' # fill missing columns at the end with empty strings
			if entry["id"] != '': # convert ID# column into an int
				cur_id = int(entry["id"])
				entry["id"] = cur_id
				if cur_id > max_id: # remember the maximum ID#
					max_id = cur_id
				del cur_id
			else:
				entry["id"] = None
			if entry["set"].split(',')[0] in ("core", "ext", "mix", "spec"):
				tables.table[(comp,ident)]=entry
			del entry
		del comp, ident
	# the legend is uninteresting, close the tracking table now
	file.close()
	tables.ids = [None for i in range(0, max_id+1)]
