#!/usr/bin/python
#encoding: utf8
"""
this file contains parsing rules for graphics evaluation
"""

from tokens import tokens
import tree

def p_graphics1(p):
	"graphics : target"
	p[0] = p[1]

def p_condition1(p):
	"condition : value"
	p[0] = p[1]

def p_condition2(p):
	"condition : condition ',' value"
	p[0] = p[1]+p[3]

def p_value(p):
	"value : ranges ':' target"
	p[0] = [(x, p[3]) for x in p[1]]

def p_ranges1(p):
	"ranges : range"
	p[0] = p[1]

def p_ranges2(p):
	"ranges : ranges '|' range"
	p[0] = p[1]+p[3]

def p_range(p):
	"""
	range : literal
	range : literal  '.' '.' literal
	range : '>' '=' literal
	range : '<' '=' literal
	range : '>' literal
	range : '<' literal
	"""
	p[0] = [''.join(p[1:])]

def p_target1(p):
	"target : literal"
	p[0] = tree.Leaf(p[1])

def p_target2(p):
	"target : LITERAL ':' '(' condition ')'"
	p[0] = tree.Tree(p[1], *p[4])

def p_target3(p):
	"target : LITERAL '<' list '>' '(' condition ')'"
	if p[1] == "template":
		p[0] = tree.Template(*p[3], args=p[6])
	else:
		p[0] = tree.shape(p[1])(*p[3], args=p[6])

def p_list1(p):
	"list :"
	p[0] = []

def p_list2(p):
	"list : literal"
	p[0] = [p[1]]

def p_list3(p):
	"list : list ',' literal"
	p[1].append(p[3])
	p[0] = p[1]

def p_literal(p):
	"""
	literal : LITERAL
	literal : NUMBER
	"""
	p[0]=p[1]
