#!/usr/bin/python
#encoding: utf8
"""
This provides the shapes for pixa image generation
"""

import tables
from helper import shade

class Shape(object):
	def __init__(self, height, args):
		self.height = int(height)
		self.colours = []
		for i in range(0,self.height):
			self.colours.append(None)
		for (value, target) in args:
			if ".." in value:
				value = map(int, value.split('..'))
				value[1]+=1
			elif value.startswith("<="):
				value = [0, int(value[2:])+1]
			elif value.startswith(">="):
				value = [int(value[2:]), self.length]
			elif value.startswith(">"):
				value = [int(value[1:])+1, self.length]
			elif value.startswith("<"):
				value = [0, int(value[1:])]
			else:
				value = [int(value), int(value)+1]
			for v in range(value[0], value[1]):
				if '+' in target.value:
					self.colours[v] = shade(*((lambda (a,b):(tables.colours.get(a),int(b)))(target.value.split('+'))))
				elif '-' in target.value:
					self.colours[v] = shade(*((lambda (a,b):(tables.colours.get(a),-int(b)))(target.value.split('-'))))
				else:
					try:
						self.colours[v] = tables.colours[target.value]
					except KeyError:
						self.colours[v] = int(target.value)
		for i, colour in enumerate(self.colours):
			if colour is None:
				raise Exception("colour for height %i is missing"%(i,))
			if colour < 0 or colour > 255:
				raise Exception("colour for height %i: %i is out of range"%(i, colour))
	__repr__ = lambda self: "%s(%r, %r)"%(self.__class__.__name__,self.height, enumerate(self.colours))
	def write(self, **args):
		raise Exception("shape cannot appear outside a template")
	def evaluate(self, offset):
		raise NotImplemented("Shape needs to override this function")
	def evaluate_front(self):
		raise NotImplemented("Shape needs to override this function")
	def evaluate_back(self):
		raise NotImplemented("Shape needs to override this function")

class Box(Shape):
	def evaluate(self, offset):
		ret = {}
		for index in [16*i+1 for i in range(0,12)]:
			ret[index+offset]=[]
			for i in range(0, self.height):
				ret[index+offset].append((0,-i,self.colours[i]))
		return ret
	def evaluate_front(self):
		ret = {}
		for index in range(231,231+12):
			ret[index]=[]
			for i in range(0, self.height):
				#make front and back one shade darker
				ret[index].append((0,-i,shade(self.colours[i],-1)))
		return ret
	def evaluate_back(self):
		ret = {}
		for index in range(243,243+12):
			ret[index]=[]
			for i in range(0, self.height):
				#make front and back one shade darker
				ret[index].append((0,-i,shade(self.colours[i],-1)))
		return ret

class Open(Shape):
	def evaluate(self, offset):
		ret = {}
		#outside
		for index in (1, 16*11+1):
			ret[index+offset]=[]
			for i in range(0, self.height):
				ret[index+offset].append((0,-i,self.colours[i]))
		#inside
		for index in [16*i+1 for i in range(1,11)]:
			#TODO: height of cargo
			ret[index+offset]=[(0,0,shade(self.colours[self.height-1],-2))]
		return ret
	def evaluate_front(self):
		ret = {}
		for index in range(231,231+12):
			ret[index]=[]
			for i in range(0, self.height):
				#make front and back one shade darker
				ret[index].append((0,-i,shade(self.colours[i],-1)))
		return ret
	def evaluate_back(self):
		ret = {}
		for index in range(243,243+12):
			ret[index]=[]
			for i in range(0, self.height):
				#make front and back one shade darker
				ret[index].append((0,-i,shade(self.colours[i],-1)))
		return ret

class Debug(Shape):
	def evaluate(self, offset):
		ret = {}
		for index in [16*i+1 for i in range(0,12)]:
			ret[index+offset]=[(0, 0, 1)]
		return ret
	def evaluate_front(self):
		ret = {}
		for index in range(231,231+12):
			ret[index]=[(0, 0, 1)]
		return ret
	def evaluate_back(self):
		ret = {}
		for index in range(243,243+12):
			ret[index]=[(0, 0, 1)]
		return ret

def shape(name):
	"factory function for shapes"
	#return Debug
	return {
		"box"      : Box,
		"open"     : Open,
		}[name]
