#!/usr/bin/python
#encoding: utf8
"""
This file contains the functions writing the non-engine files
"""
import tables
import project_config
import helper
from error import warning
from parser import parse

def src_deps(file, deps):
	for dep in deps:
		file.write(project_config.project_short_name + ".nfo: %s\n"%(dep,))

def gfx_deps(file, deps):
	for src, dep in deps:
		file.write("%s: %s\n"%(src,dep))

def copy(in_file, out_file):
	"""line-by-line copy of the input file"""
	for line in in_file:
		out_file.write(line)
	in_file.close()

def strings(file, auto_strings):
	"""write auto_strings to file"""
	for i, string in enumerate(auto_strings):
		file.write('STR_AUTO_%d :%s\n'%(i, string.replace('{','{{}')))

def name(files, (comp, ident), long_name, values, **args):
	for _, file in files:
		file.write('STR_NAME_ENG_%s_%s :%s\n'%(comp,ident,long_name))

def table_header(file, titles, width):
	for i, title in enumerate(titles):
		file.write(title)
		file.write(' '*(width[i]-len(title)+1))
	file.write('\n')

def availability(file, dfile, avails, companies):
	for epoch in companies:
		max_value = 1
		regions = {}
		params = [("VAL_EPOCH_NONE", "STR_PARAM_COMPANY_NONE"), ("VAL_EPOCH_ALL", "STR_PARAM_EPOCH_ALL")]
		for ident, name, value, region, rvalue in companies[epoch].itervalues():
			dfile.write('#define %s %s\n'%(ident, value))
			params.append((ident, name))
			if value > max_value:
				max_value = value
			if region not in regions:
				rident = "VAL_EPOCH%s_REGION_%s"%(epoch, region)
				rname = "STR_PARAM_EPOCH_REGION_%s"%(region,)
				if value != rvalue:
					regions[region] = (rident, rname)
				dfile.write('#define %s %s\n'%(rident, rvalue))
		dfile.write('#define PARAM_EPOCH%s 	param {\\\n'%(epoch,))
		dfile.write('		param_epoch%s {\\\n'%(epoch,))
		dfile.write('			type:    int;\\\n')
		dfile.write('			name:    string(STR_PARAM_EPOCH%s);\\\n'%(epoch,))
		dfile.write('			desc:    string(STR_PARAM_EPOCH%s_DESC);\\\n'%(epoch,))
		dfile.write('			min_value: 0;\\\n')
		dfile.write('			max_value: %s;\\\n'%(max_value,))
		dfile.write('			def_value: 1;\\\n')
		dfile.write('			names: {\\\n')
		for rident, rname in regions.itervalues():
			dfile.write('			%s: string(%s);\\\n'%(rident, rname))
		for ident, name in params:
			dfile.write('			%s: string(%s);\\\n'%(ident, name))
		dfile.write('			};\\\n')
		dfile.write('		}\\\n')
		dfile.write('	}\n')
	dfile.close()
	for a in sorted(avails):
		if a.comp in companies[a.epoch]:
			file.write('%s = (%s) && (%s || %s || %s);\n'%(
				a, 
				{'core':'param_amount >= VAL_AMOUNT_CORE','ext':'param_amount == VAL_AMOUNT_ALL'}[a.set],
				'param_epoch%s == VAL_EPOCH_ALL'%(a.epoch,),
				'param_epoch%s == VAL_EPOCH%s_REGION_%s'%(a.epoch, a.epoch, companies[a.epoch][a.comp][3]),
				'param_epoch%s == %s'%(a.epoch, companies[a.epoch][a.comp][0])))
		else:
			file.write('%s = 0;\n'%(a,))
	file.close()

def table_entry(file, (comp, ident), values, **args):
	for i, title in enumerate(tables.titles):
		file.write('%s'%(values[tables.columns.get(title, title)],))
		file.write(' '*(tables.width[i]-len('%s'%(values[tables.columns.get(title, title)]),)+1))
	file.write('\n')

def global_callbacks(file, caps):
	for cap in caps:
		if cap == 0: continue
		file.write('switch (FEAT_TRAINS, SELF, capacity_cb_%s_switch, cargo_type_in_veh) {\n'%(cap,))
		file.write('	TOUR: return %s/2;\n'%(cap,))
		file.write('	VALU: return %s/4;\n'%(cap,))
		file.write('	GOLD: return %s/4;\n'%(cap,))
		file.write('	DIAM: return %s/4;\n'%(cap,))
		file.write('	return %s;\n'%(cap,))
		file.write('}\n')
	file.close()

def dummy_code(file, auto_strings, fixed_strings, caps):
	# dummy callbacks to enforce using all strings, and global callbacks keep their ID allocated
	file.write('comment("__DUMMY_CODE_BEGIN__");\n')
	file.write('#undef DUMMY_CALLBACK\n')
	file.write('#ifdef VEH_ID\n')
	file.write('#define DUMMY_CALLBACK VEH_ID(dummy_callback)\n')
	file.write('#else\n')
	file.write('#define DUMMY_CALLBACK dummy_callback\n')
	file.write('#endif')
	file.write('\n')
	file.write('switch(FEAT_TRAINS, SELF, DUMMY_CALLBACK, 0xFFFFFFFF) {\n')
	for i in range(len(auto_strings)):
		file.write('	%d : string(STR_AUTO_%d);\n'%(i, i))
	for i, string in enumerate(fixed_strings):
		file.write('	%d : string(%s);\n'%(i + len(auto_strings), string))
	for i, cap in enumerate(caps):
		if cap == 0: continue
		file.write('	%d : capacity_cb_%s_switch;\n'%(i + len(auto_strings) + len(fixed_strings), cap))
	file.write('	inv_dummy_group;\n')
	file.write('}\n')
	file.write('\n')
	file.write('item(FEAT_TRAINS, dummy_vehicle, 0xFFFF) {\n')
	file.write('	graphics { default: DUMMY_CALLBACK; }\n')
	file.write('}\n')
	file.write('comment("__DUMMY_CODE_END__");\n')
	file.close()

def templates(file, length, slices, offsets):
	center = (len(slices)-1)/2
	center_start = sum(slices[:center])
	assert length == sum(slices)
	# spritesets
	# normal view
	sl_start = 0
	for i in range(-center, center+1):
		sl_length = slices[center+i]
		file.write("#if SLICE == %d\n"%(i,))
		for color_depth in project_config.graphical_features:
			for zoom_level in project_config.graphical_features[color_depth]:
				file.write('#ifdef GFX_%s_%s\n'%(color_depth,zoom_level))
				file.write(helper.view_normal("slice_set", length, 0, sl_start, sl_start+sl_length, offsets["view_normal"], color_depth, zoom_level))
				file.write("#endif\n")
		file.write("#endif\n")
		sl_start+=sl_length
	# gui view
	sl_start = 0
	for i in range(-center, center+1):
		sl_length = slices[center+i]
		file.write("#if SLICE == %d\n"%(i,))
		for color_depth in project_config.graphical_features:
			for zoom_level in project_config.graphical_features[color_depth]:
				file.write('#ifdef GFX_%s_%s\n'%(color_depth,zoom_level))
				file.write(helper.view_normal("gui_set", length, sl_start, 0, 16, offsets["view_normal"], color_depth, zoom_level))
				file.write("#endif\n")
		file.write("#endif\n")
		sl_start+=sl_length
	file.write("#if SLICE == 0\n")
	# complete view
	for color_depth in project_config.graphical_features:
		for zoom_level in project_config.graphical_features[color_depth]:
			file.write('#ifdef GFX_%s_%s\n'%(color_depth,zoom_level))
			file.write(helper.view_normal("set", length, center_start, 0, 16, offsets["view_normal"], color_depth, zoom_level))
			file.write("#endif\n")
	# slanted views
	for color_depth in project_config.graphical_features:
		for zoom_level in project_config.graphical_features[color_depth]:
			file.write('#ifdef GFX_%s_%s\n'%(color_depth,zoom_level))
			for view in ("view_left_2", "view_left_1", "view_right_2", "view_right_1"):
				file.write(eval("helper.%s"%(view))("%s_set"%(view,), length, center_start, 0, 16, offsets[view], color_depth, zoom_level))
			file.write("#endif\n")
	file.write("#endif\n")
	# spritegroups
	for view in ("slice", "gui"):
		file.write("spritegroup VEH_ID(%s_group) {\n"%(view,))
		file.write("	loading: VEH_ID(%s_set);\n"%(view,))
		file.write("	loaded: VEH_ID(%s_set);\n"%(view,))
		file.write("}\n")
	file.write("#if SLICE == 0\n")
	file.write("spritegroup VEH_ID(group) {\n")
	file.write("	loading: VEH_ID(set);\n")
	file.write("	loaded: VEH_ID(set);\n")
	file.write("}\n")
	for view in ("view_left_2", "view_left_1", "view_right_2", "view_right_1"):
		file.write("spritegroup VEH_ID(%s_group) {\n"%(view,))
		file.write("	loading: VEH_ID(%s_set);\n"%(view,))
		file.write("	loaded: VEH_ID(%s_set);\n"%(view,))
		file.write("}\n")
	file.write("#endif\n")
	# magic slice switches
	file.write("""
switch(FEAT_TRAINS, SELF, VEH_ID(height_switch), ((var[0x62, 24, 0x000000FF, -1 - (SLICE)] - var[0x62, 24, 0x000000FF, -0 - (SLICE)]) & 0xFF) << 8 | ((var[0x62, 24, 0x000000FF, -0 - (SLICE)] - var[0x62, 24, 0x000000FF, 1 - (SLICE)]) & 0xFF)) {
	0 : VEH_ID(slice_group);
#if SLICE == 0
	VEH_ID(group);
#else
	inv_dummy_group;
#endif
}

switch(FEAT_TRAINS, SELF, VEH_ID(direction_switch), ((var[0x62, 0, 0x0000000F, -1 - (SLICE)] - var[0x62, 0, 0x0000000F, -0 - (SLICE)]) & 0xF) << 8 | ((var[0x62, 0, 0x0000000F, -0 - (SLICE)] - var[0x62, 0, 0x0000000F, 1 - (SLICE)]) & 0xF)) {
#if SLICE == 0
	0x00000000: VEH_ID(height_switch);
	0x00000001: VEH_ID(view_left_2_group);
	0x00000002: VEH_ID(view_left_2_group);
	0x00000E00: VEH_ID(view_left_1_group);
	0x00000F00: VEH_ID(view_left_1_group);
	0x0000000E: VEH_ID(view_right_2_group);
	0x0000000F: VEH_ID(view_right_2_group);
	0x00000100: VEH_ID(view_right_1_group);
	0x00000200: VEH_ID(view_right_1_group);
	VEH_ID(group);
#else
	0x00000000: VEH_ID(height_switch);
	inv_dummy_group;
#endif
}

switch(FEAT_TRAINS, SELF, VEH_ID(gfx_switch), extra_callback_info1 & 0xFF) {
	0 : VEH_ID(direction_switch);
	VEH_ID(gui_group);
}
""")

def engine_sort_header(file):
	file.write('#include "src/headers.pnml"\n')
	file.write('sort(FEAT_TRAINS, [\n')

def engine_sort_entry(file, key, slices, **args):
	file.write('	%d,\n'%(slices[0][0],))

def engine_sort_footer(file):
	file.write(']);\n')
	file.close()

def engines_header(file):
	file.write('#!/bin/sh\n')
	file.write("awk 'BEGIN {head=1; dummy=0} /__DUMMY_CODE_BEGIN__/{dummy=1} {if ( head && !dummy) print $0} /__DUMMY_CODE_END__/{dummy=0} /__END_HEADERS__/{head=0}' src/headers.nfo > cets.nfo\n")

def engines_entry(file, key, filename, **args):
	file.write("echo '# %s' >> cets.nfo\n"%(filename,))
	file.write("awk 'BEGIN {head=1; dummy=0} /__DUMMY_CODE_BEGIN__/{dummy=1} {if (!head && !dummy) print $0} /__DUMMY_CODE_END__/{dummy=0} /__END_HEADERS__/{head=0}' '%s' >> cets.nfo\n"%(filename.replace('.gnml.new','.onfo'),))
	file.write("echo '# %s' >> cets.nfo\n"%(filename,))

def engines_footer(file):
	file.write("awk 'BEGIN {head=1; dummy=0} /__DUMMY_CODE_BEGIN__/{dummy=1} {if (!head && !dummy) print $0} /__DUMMY_CODE_END__/{dummy=0} /__END_HEADERS__/{head=0}' 'src/engine_sort.onfo' >> cets.nfo\n")
