#!/usr/bin/python
#encoding: utf8
"""
This file contains the functions writing the engine files
"""
import tables
import project_config
import helper
from error import warning
from parser import parse

def graphics(deps_table, (comp, ident), vehicle_file, length, slices, sl_pos, graphics, graphics_file, recolour, values, **args):
	file = vehicle_file
	new_graphics = []
	new_recolour = []
	if graphics:
		for i, (sl_id, sl_length) in enumerate(slices):
			new_graphics.append(parse(graphics).write(file, "%s_%s_%s"%(comp, ident, i), "%s/%s_%s"%(comp, graphics_file or ident, length), "gfx_front_vehicle_%s.gnml"%(length,), slice = (i-sl_pos)))
			if recolour:
				new_recolour.append(parse(recolour).write(file, "%s_%s_%s"%(comp, ident, i), postfix = "recolour", recolour = True, slice = (i-sl_pos)))
			else:
				new_recolour = None
	else:
		for i, (sl_id, sl_length) in enumerate(slices):
			file_name_prefix = "src/gfx/template_%s"%(length,)
			for color_depth in project_config.graphical_features:
				for zoom_level in project_config.graphical_features[color_depth]:
					filename = '%s_%s_%s.png'%(file_name_prefix,color_depth,zoom_level)
					if (file.name.replace(".gnml.new",".onml"), filename) not in deps_table:
						deps_table.append((file.name.replace(".gnml.new",".onml"), filename))
					file.write('#define GFX_%s_%s "%s"\n'%(color_depth,zoom_level, filename))
			file.write('#define VEH_ID(...) %s_%s_%s ## __VA_ARGS__\n'%(comp, ident, i))
			file.write('#define SLICE %s\n'%(i-sl_pos,))
			file.write('#include "src/gfx_front_vehicle_%s.gnml"\n'%(length,))
			new_graphics.append(('%s_%s_%sgfx_switch'%(comp,ident,i), '%s_%s_%sgui_group\n'%(comp,ident,i)))
			file.write('#undef SLICE\n')
			file.write('#undef VEH_ID\n')
			for color_depth in project_config.graphical_features:
				for zoom_level in project_config.graphical_features[color_depth]:
					file.write('#undef GFX_%s_%s\n'%(color_depth,zoom_level))
		new_recolour = None
	values["graphics"] = new_graphics
	values["recolour"] = new_recolour

def properties(key, slices, sl_pos, values, **args):
	for i, (sl_id, sl_length) in enumerate(slices):
		properties_slice(i, sl_id, sl_length, key, **values)
		if i == 0:
			properties_master(i, sl_id, sl_length, key, **values)

def properties_slice(sl_index, sl_id, sl_length, (comp, ident), vehicle_file, values, **args):
	file = vehicle_file
	file.write('#define VEH_ID(...) %s_%s_%s ## __VA_ARGS__\n'%(comp, ident, sl_index))
	file.write('\n')
	file.write('item(FEAT_TRAINS, VEH_ID(), %d) {\n'%(sl_id,))
	file.write('	property {\n')
	file.write('		sprite_id:                      SPRITE_ID_NEW_TRAIN;\n')
	file.write('		length:                         %s;\n'%(sl_length,))
	file.write('		name:                           0;\n')
	file.write('		climates_available:             NO_CLIMATE;\n')
	file.write('\n')
	file.write('		introduction_date:              0;\n')
	file.write('		vehicle_life:                   0;\n')
	file.write('		retire_early:                   0;\n')
	file.write('		model_life:                     0;\n')
	file.write('		reliability_decay:              0;\n')
	file.write('\n')
	file.write('		track_type:                     0;\n')
	file.write('		speed:                          0;\n')
	file.write('		power:                          0;\n')
	file.write('		weight:                         0;\n')
	file.write('		tractive_effort_coefficient:    0;\n')
	file.write('		air_drag_coefficient:           0;\n')
	file.write('\n')
	file.write('		loading_speed:                  0;\n')
	file.write('		cargo_capacity:                 0;\n')
	file.write('		refittable_cargo_classes:       0;\n')
	file.write('		non_refittable_cargo_classes:   0;\n')
	file.write('		cargo_allow_refit:              [];\n')
	file.write('		cargo_disallow_refit:           [];\n')
	file.write('		cargo_age_period:               0;\n')
	file.write('\n')
	file.write('		visual_effect_and_powered:      visual_effect_and_powered(VISUAL_EFFECT_DISABLE, 0, DISABLE_WAGON_POWER);\n')
	file.write('		engine_class:                   0;\n')
	file.write('\n')
	file.write('		dual_headed:                    0;\n')
	file.write('		extra_power_per_wagon:          0;\n')
	file.write('		extra_weight_per_wagon:         0;\n')
	file.write('		misc_flags:                     0;\n')
	file.write('\n')
	file.write('		cost_factor:                    0;\n')
	file.write('		refit_cost:                     0;\n')
	file.write('		running_cost_base:              0;\n')
	file.write('		running_cost_factor:            0;\n')
	file.write('\n')
	file.write('		ai_engine_rank:                 0;\n')
	file.write('		ai_special_flag:                0;\n')
	file.write('\n')
	file.write('		bitmask_vehicle_info:           0;\n')
	file.write('	}\n')
	file.write('}\n')
	file.write('#undef VEH_ID\n')

def properties_master(sl_index, sl_id, sl_length, (comp, ident), vehicle_file, translatable_name, availability, max_speed, track_type_classes, track_type_speed, track_type_default, company, epoch, set, intro_date, power, weight, traction_type, max_te, capacity, loading_speed, allowed, forbidden, add_cargos, rem_cargos, cost_factor, vehicle_life, retire_early, model_life, run_cost_factor, usage, ai_special_flag, values, **args):
	file = vehicle_file
	file.write('#define VEH_ID(...) %s_%s_%s ## __VA_ARGS__\n'%(comp, ident, sl_index))
	file.write('#define %s_%s %s_%s_%s\n'%(comp, ident, comp, ident, sl_index))
	file.write('\n')
	file.write('item(FEAT_TRAINS, VEH_ID(), %d) {\n'%(sl_id,))
	file.write('	property {\n')
	if translatable_name:
		file.write('		name:                           string(STR_NAME_ENG_%s_%s,%s);\n'%(comp,ident,translatable_name))
	else:
		file.write('		name:                           string(STR_NAME_ENG_%s_%s);\n'%(comp,ident))
	file.write('		climates_available:             (%s) ? ALL_CLIMATES : NO_CLIMATE;\n'%(availability,))
	file.write('\n')
	file.write('		introduction_date:              date(%s,1,1);\n'%((lambda y: (1, warning("%s %s: Has no intro date set"%(comp,ident)))[0] if y=='' else y)(intro_date),))
	file.write('		vehicle_life:                   %s;\n'%(vehicle_life,))
	file.write('		retire_early:                   %s;\n'%(retire_early,))
	file.write('		model_life:                     %s;\n'%(model_life,))
	file.write('		reliability_decay:              20;\n')
	file.write('\n')
	file.write('		track_type:                     used_trackset == VAL_TRACKSET_CLASSES ? %s:\n'%(track_type_classes,))
	file.write('		                                used_trackset == VAL_TRACKSET_SPEED   ? %s:\n'%(track_type_speed,))
	file.write('		                                                                        %s;\n'%(track_type_default,))
	file.write('		speed:                          %d nfo;\n'%(max_speed,))
	file.write('		power:                          %d hp;\n'%(round(1/0.745*power),))
	file.write('		weight:                         %s tons;\n'%(weight,))
	file.write('		tractive_effort_coefficient:    %s;\n'%('0' if traction_type in ["Wagon","Wagon/Narrow"] else '1.0*%s/%s/10'%(max_te,weight),))
	file.write('\n')
	file.write('		loading_speed:                  %s;\n'%(loading_speed,))
	file.write('		cargo_capacity:                 %s;\n'%(capacity,))
	file.write('		refittable_cargo_classes:       %s;\n'%(allowed,))
	file.write('		non_refittable_cargo_classes:   %s;\n'%(forbidden,))
	file.write('		cargo_allow_refit:              %s;\n'%(add_cargos,))
	file.write('		cargo_disallow_refit:           %s;\n'%(rem_cargos,))
	file.write('		cargo_age_period:               %s;\n'%(
		"DEFAULT_CARGO_AGING_PERIOD * 2" if usage in ["express","express/passenger","local/express","passenger,express","refrigerated freight"] else
		"DEFAULT_CARGO_AGING_PERIOD * 1.5" if usage in ["sheltered bulk","liquids","livestock"] else	
		"DEFAULT_CARGO_AGING_PERIOD / 1.5" if usage in ["local","local (passenger)","passenger/local","local/passenger"] else
		"DEFAULT_CARGO_AGING_PERIOD / 2" if usage in ["commuter","local/commuter","commuter/express"] else
		"DEFAULT_CARGO_AGING_PERIOD",
		))
	file.write('\n')
	file.write('		visual_effect_and_powered:      visual_effect_and_powered(VISUAL_EFFECT_%s, 0, DISABLE_WAGON_POWER);\n'%(
		(lambda x:
			"STEAM"    if x in ["Steam"] else 
			"DIESEL"   if x in ["Diesel"] else 
			"ELECTRIC" if x in ["Electric"] else 
			"DISABLE"  if x in ["Battery", "3rd Rail"] else 
			"DISABLE"
		)(traction_type.split("/")[0]),))
	file.write('		engine_class:                   ENGINE_CLASS_STEAM;\n')
	file.write('\n')
	file.write('		dual_headed:                    0;\n')
	file.write('		extra_power_per_wagon:          0;\n')
	file.write('		extra_weight_per_wagon:         0;\n')
	file.write('		misc_flags:                     0;\n')
	file.write('\n')
	file.write('		cost_factor:                    %d;\n'%(round(cost_factor),))
	file.write('		refit_cost:                     10;\n')
	file.write('		running_cost_base:              RUNNING_COST_%s;\n'%(
		(lambda x:
			"STEAM"    if x in ["Steam","Wagon"] else 
			"DIESEL"   if x in ["Diesel","Battery"] else  
			"ELECTRIC"
		)(traction_type.split("/")[0]),))
	file.write('		running_cost_factor:            %s;\n'%(run_cost_factor,))
	file.write('\n')
	file.write('		ai_engine_rank:                 0;\n')
	file.write('		ai_special_flag:                %d;\n'%(ai_special_flag,))
	file.write('\n')
	file.write('		bitmask_vehicle_info:           0;\n')
	file.write('	}\n')
	file.write('}\n')
	file.write('\n')
	file.write('#undef VEH_ID\n')

def articulation((comp, ident), vehicle_file, articulation, **args):
	file = vehicle_file
	if articulation:
		file.write('#define VEH_ID(...) %s_%s_%s ## __VA_ARGS__\n'%(comp, ident, 0))
		file.write('#define ARTIC VEH_ID(cb_articulate_switch)\n')
		file.write('switch(FEAT_TRAINS, SELF, VEH_ID(cb_articulate_switch), extra_callback_info1) {\n',)
		for offset, sl_id in enumerate(articulation):
			file.write('	%d: return %s;\n'%(offset, sl_id))
		file.write('	return CB_RESULT_NO_MORE_ARTICULATED_PARTS;\n')
		file.write('}\n')
		file.write('#undef VEH_ID\n')

def purchase_text((comp, ident), vehicle_file, axle_scheme_string, usage_string, track_class_string, track_speed_string, track_type_string, **args):
	file = vehicle_file
	file.write('#define VEH_ID(...) %s_%s_%s ## __VA_ARGS__\n'%(comp, ident, 0))
	file.write('switch(FEAT_TRAINS, SELF, VEH_ID(text_switch_classes), [STORE_TEMP(string(%s) | string(%s) << 16, 0x100), STORE_TEMP(string(STR_TRACK_TYPES_CLASSES) | string(%s) << 16, 0x101), STORE_TEMP(string(%s), 0x102), 0]) {\n'%(axle_scheme_string, usage_string, track_class_string, track_type_string))
	file.write('	return string(STR_ENG_DESCRIPTION);\n')
	file.write('}\n')
	file.write('\n')
	file.write('switch(FEAT_TRAINS, SELF, VEH_ID(text_switch_speed),   [STORE_TEMP(string(%s) | string(%s) << 16, 0x100), STORE_TEMP(string(STR_TRACK_TYPES_SPEED)   | string(%s) << 16, 0x101), STORE_TEMP(string(%s), 0x102), 0]) {\n'%(axle_scheme_string, usage_string, track_speed_string, track_type_string))
	file.write('	return string(STR_ENG_DESCRIPTION);\n')
	file.write('}\n')
	file.write('\n')
	file.write('switch(FEAT_TRAINS, SELF, VEH_ID(text_switch_default), [STORE_TEMP(string(%s) | string(%s) << 16, 0x100), STORE_TEMP(string(STR_TRACK_TYPES_DEFAULT) | string(%s) << 16, 0x101), 0]) {\n'%(axle_scheme_string, usage_string, track_type_string))
	file.write('	return string(STR_ENG_DESCRIPTION);\n')
	file.write('}\n')
	file.write('\n')
	file.write('switch(FEAT_TRAINS, SELF, VEH_ID(text_switch), used_trackset) {\n')
	file.write('	VAL_TRACKSET_CLASSES  : VEH_ID(text_switch_classes);\n')
	file.write('	VAL_TRACKSET_SPEED    : VEH_ID(text_switch_speed);\n')
	file.write('	                        VEH_ID(text_switch_default);\n')
	file.write('}\n')
	file.write('#undef VEH_ID\n')

def callbacks((comp, ident), vehicle_file, sl_pos, graphics, recolour, slices, capacity, loading_speed, custom_callbacks, traction_type, intro_date, run_cost_factor, values, **args):
	file = vehicle_file
	if capacity != 0:
		file.write('#define CAPACITY capacity_cb_%s_switch\n'%(capacity,))
		file.write('#define LOADING_SPEED capacity_cb_%s_switch\n'%(loading_speed,))
	for i, (sl_id, sl_length) in enumerate(slices):
		file.write('#define GFX_%s %s\n'%(i, graphics[i][0]))
		file.write('#define GUI_%s %s\n'%(i, graphics[i][1]))
		if recolour is not None:
			file.write('#define RECOLOUR_%s %s\n'%(i, recolour[i][0]))
			file.write('#define RECOLOUR_GUI_%s %s\n'%(i, recolour[i][1]))
	file.write('#define TEXT VEH_ID(text_switch)\n')
	file.write('#define SOUND_EFFECT (extra_callback_info1 == SOUND_EVENT_START) ? SOUND_%s : CB_RESULT_NO_SOUND\n'%("TRAIN" if traction_type in ["Steam","Steam/Narrow"] or intro_date < 1950 else "TRAIN_HORN"))
	file.write('#define RUN_COST (vehicle_is_stopped || vehicle_is_in_depot) ? %d : %d\n'%(run_cost_factor/8,run_cost_factor))
	file.write('#define RUN_COST_PURCHASE %d\n'%(run_cost_factor,))
	if custom_callbacks:
		file.write('#define VEH_ID(...) %s_%s_0 ## __VA_ARGS__\n'%(comp, ident))
		file.write('#include "src/%s/callbacks_%s.pnml"\n'%(comp, custom_callbacks,))
		file.write('#undef VEH_ID\n')
	for i, (sl_id, sl_length) in enumerate(slices):
		file.write('\n')
		file.write('#define VEH_ID(...) %s_%s_%s ## __VA_ARGS__\n'%(comp, ident, i))
		file.write('item(FEAT_TRAINS, VEH_ID()) {\n')
		file.write('	graphics {\n')
		callbacks_slice(i, vehicle_file)
		if i == 0:
			callbacks_master(vehicle_file)
		file.write('	}\n')
		file.write('}\n')
		file.write('#undef VEH_ID\n')
		file.write('\n')
	for i, (sl_id, sl_length) in enumerate(slices):
		file.write('#undef GFX_%s\n'%(i,))
		file.write('#undef GUI_%s\n'%(i,))
		file.write('#undef RECOLOUR_%s\n'%(i,))
		file.write('#undef RECOLOUR_GUI_%s\n'%(i,))
	file.write('#undef CAPACITY\n')
	file.write('#undef LOADING_SPEED\n')
	file.write('#undef TEXT\n')
	file.write('#undef SOUND_EFFECT\n')
	file.write('#undef RUN_COST\n')
	file.write('#undef RUN_COST_PURCHASE\n')
	file.write('#undef ARTIC\n')
	file.write('#undef CUSTOM\n')


def callbacks_slice(sl_index, vehicle_file):
	file = vehicle_file
	file.write('		default: GFX_%d; purchase: GUI_%d;\n'%(sl_index, sl_index))
	file.write('#ifdef RECOLOUR_%d\n'%(sl_index))
	file.write('		colour_mapping : RECOLOUR_%d; purchase_colour_mapping : RECOLOUR_GUI_%d;\n'%(sl_index, sl_index))
	file.write('#endif\n')

def callbacks_master(vehicle_file):
	file = vehicle_file
	file.write('#ifdef CAPACITY\n')
	file.write('		cargo_capacity: CAPACITY;\n')
	file.write('		loading_speed: LOADING_SPEED;\n')
	file.write('#endif\n')
	file.write('		additional_text: TEXT;\n')
	file.write('		sound_effect: SOUND_EFFECT;\n')
	file.write('		running_cost_factor: RUN_COST;\n')
	file.write('		purchase_running_cost_factor: RUN_COST_PURCHASE;\n')
	file.write('#ifdef ARTIC\n')
	file.write('		articulated_part: ARTIC;\n')
	file.write('#endif\n')
	file.write('#ifdef CUSTOM\n')
	file.write('CUSTOM\n')
	file.write('#endif\n')

def vehicle_header(files, files_table, (comp, ident), values, graphics_file, **args):
	if graphics_file:
		file = files.vehicle_file(comp, graphics_file, files_table)
	else:
		file = files.vehicle_file(comp, ident, files_table)
	file.write('#include "src/headers.pnml"\n')
	values["filename"] = file.name
	values["vehicle_file"] = file

def vehicle_footer((comp, ident), vehicle_file, **args):
	vehicle_file.write('#undef ARTIC\n')
	vehicle_file.write('#define VEH_ID(...) %s_%s ## __VA_ARGS__\n'%(comp, ident))
	vehicle_file.write('#include "src/dummy_code.gnml"\n')
	vehicle_file.write('#undef VEH_ID\n')
	vehicle_file.close()
